@@ -1,4 +1,10 @@
+# S3method for "tocher"
 tocher <-
+function(d, algorithm = c("original", "sequential")) UseMethod("tocher")
+
+# ---------------------------------
+# method for object of class dist
+tocher.dist <-
 function(d, algorithm = c("original", "sequential"))
 {
    if (!inherits(d, "dist"))
@@ -89,19 +95,80 @@ function(d, algorithm = c("original", "sequential"))
    }
    nopc <- sapply(g, length)
    dc <- distClust(as.dist(d), nopc, unlist(g))
-   out <- list(clusters = g, class = class, 
-      criterion = criterion, distClust = dc, 
+   out <- list(call = match.call(), 
+      algorithm = algorithm,
+      clusters = g, 
+      class = class, 
+      criterion = criterion, 
+      distClust = dc, 
       d = as.dist(d))
    class(out) <- "tocher"
    return(out)
 }
 
-# -------------------------------------------
+# ---------------------------------
 # print method
 print.tocher <- 
-function (x, digits = 4L, quote = TRUE, ...) 
+function (x, ...) 
 {
    cat("\n          Tocher's Clustering \n\n")
-   print(x$clusters)
+   n <- attr(x$d, "Size")
+   cat("Call: ")
+      print(x$call)
+   cat("\nCluster algorithm:", x$algorithm,
+      "\nNumber of objects:", n,
+      "\nNumber of clusters:", length(x$clusters), "\n")
+   if (n < 25) {
+      id <- maxmat(x$distClust)
+      cat("Most contrasting clusters: ", id[1], " and ", id[2],
+          ", with \n   average intercluster distance: ", 
+             x$distClust[id[1], id[2]], "\n\n", sep = "")
+      print(x$clusters)
+   } else {
+      cat("Most contrasting clusters (first 4 objects):\n")
+      id <- maxmat(x$distClust)
+      nopc <- sapply(x$clusters[id], length)
+      rest <- max(nopc) - nopc
+      dd <- data.frame(1:max(nopc))
+      for(i in 1:length(nopc)) {
+         dd[, i] <- c(x$clusters[[id[i]]], rep("", rest[id[i]]))
+      }
+      colnames(dd) <- paste(names(x$clusters[id]), 
+         " (size ", nopc, ")", sep = "")
+      print(head(dd, 4))
+      cat("... with average intercluster distance:", 
+         x$distClust[id[1], id[2]], "\n")
+   }
    invisible(x)
 }
+
+# -------------------------------------------
+# cophenetic method
+coph.tocher <- function(x) {
+   .Deprecated("cophenetic", package = "biotools")
+   cophenetic.tocher(x=x)
+}
+
+cophenetic.tocher <-
+function(x)
+{
+   dc <- x$distClust
+   d <- x$d
+   ord <- attr(d, "Labels")
+   lab <- unlist(x$clusters)
+   n <- length(lab)
+   nc <- length(x$clusters)
+   nopc <- sapply(x$clusters, length)
+   cl <- rep(1:nc, nopc)
+   coph <- matrix(0, n, n)
+   dimnames(coph) <- list(lab, lab)
+   for(j in 1:n) {
+      for(i in 1:n) {
+         if (i != j) {
+            coph[lab[i], lab[j]] <- dc[cl[lab == lab[i]],
+               cl[lab == lab[j]]]
+         }
+      }
+   }
+   return(as.dist(coph[ord, ord]))
+}
